""" Alignment Tools for Relative T Station to VELO Alignment """
""" S. R. Blusk, Feb 18, 2007"""

def DetElemDaughters(detElem, spacer, theName) :
#
# Note:
#  For VELO, there are 3 Conditions files: Global.xml, Modules.xml, and Detectors.xml
#  For IT & OT, they are currebtly lumped into one xml file
#  For IT: Detectors.xml;    For OT: Elements.xml
#
# This routine will hangle Velo:Global.xml;  IT: Detectors.xml; and OT: Elements.xml
#
#    print 'In makeAlignments: topDetElem = ',detElem.name()
    global myXmlString
    daughterDEs = detElem.childIDetectorElements()

    if (detElem.geometry().alignmentCondition() != None ) :
        name = detElem.geometry().alignmentCondition().name()
        if( (name[1:5] == theName) or (theName==' ') ):
            misalignFunc = performTransformation( detElem.geometry() )
            modifyCondition(detElem.geometry().alignmentCondition(),
                            misalignFunc)
            newString = writeCondition(detElem.geometry().alignmentCondition() )
            myXmlString = myXmlString + newString


    for iDE in range ( daughterDEs.size() ) :
#        if (daughterDEs[iDE].geometry().alignmentCondition() != None ) :
#            print spacer + "+--->" + daughterDEs[iDE].geometry().alignmentCondition().name()

        DetElemDaughters(daughterDEs[iDE], spacer+" |", theName)


def modifyCondition(alignCond, misalignFunc) :
    """
    apply misalignFunc to alignCond. Modifies alignCond. misalignFunc must be
    a dictionary with entries ['trans'], ['rot'], ['pivot']
    """

    trans = misalignFunc['trans']
    rot   = misalignFunc['rot']
    pivot = misalignFunc['pivot']

    thePositions = alignCond.paramAsDoubleVect("dPosXYZ")
    theRotations = alignCond.paramAsDoubleVect("dRotXYZ")

    if(myConstants.verbose == True):
        print "---> The XYZ positions are: ",thePositions[0],thePositions[1],thePositions[2]
        print "---> The XYZ rotations are: ",theRotations[0],theRotations[1],theRotations[2]
        
        print 'Modifying condition '+ alignCond.name()
        print 'XYZ Trans:  ',trans[0],trans[1],trans[2]
        print 'XYZ Rots:   ',rot[0],rot[1],rot[2]
        print 'XYZ Pivots  ',pivot[0],pivot[1],pivot[2]

    sc=alignCond.setTransformation(trans, rot, pivot)

def performTransformation(geom) :
    """
    Finds the Z position of geom in the global frame and calculates
    a shift in Z, linear in Z. Returns dictionary with keys 'trans',
    'rot', 'pivot'
    """
    part_name = geom.alignmentCondition().name()
    alignCond = geom.alignmentCondition()

    thePositions = alignCond.paramAsDoubleVect("dPosXYZ")
    theRotations = alignCond.paramAsDoubleVect("dRotXYZ")

    transform = {}
    origin = 0
    scaleFactor=0.0
    zeroPoint = XYZPoint(0.,0.,0.)
    Z = geom.toGlobal(zeroPoint).z()

    transform['trans']=std.vector('double')()
    transform['rot']=std.vector('double')()
    transform['pivot']=std.vector('double')(3)

    trans = transform['trans']
    rot   = transform['rot']


    # 1. Global VELO misalignments
        
    if (part_name == '/VeloSystem') :
        trans.push_back(myConstants.veloglo_trans[0]+thePositions[0])  # X translation
        trans.push_back(myConstants.veloglo_trans[1]+thePositions[1])  # Y translation
        trans.push_back(myConstants.veloglo_trans[2]+thePositions[2])  # Z translation
        rot.push_back(myConstants.veloglo_rot[0]+theRotations[0])    # X rotation
        rot.push_back(myConstants.veloglo_rot[1]+theRotations[1])    # Y rotation
        rot.push_back(myConstants.veloglo_rot[2]+theRotations[2])    # Z rotation

    # 2. VELO halves misalignments
    
    elif (part_name == '/VeloLeft') :
        trans.push_back(myConstants.veloleft_trans[0]+thePositions[0])  # X translation
        trans.push_back(myConstants.veloleft_trans[1]+thePositions[1])  # Y translation
        trans.push_back(myConstants.veloleft_trans[2]+thePositions[2])  # Z translation
        rot.push_back(myConstafnts.veloleft_rot[0]+theRotations[0])    # X rotation
        rot.push_back(myConstants.veloleft_rot[1]+theRotations[1])    # Y rotation
        rot.push_back(myConstants.veloleft_rot[2]+theRotations[2])    # Z rotation
        
    elif (part_name == '/VeloRight') :
        trans.push_back(myConstants.veloright_trans[0]+thePositions[0])  # X translation
        trans.push_back(myConstants.veloright_trans[1]+thePositions[1])  # Y translation
        trans.push_back(myConstants.veloright_trans[2]+thePositions[2])  # Z translation
        rot.push_back(myConstants.veloright_rot[0]+theRotations[0])    # X rotation
        rot.push_back(myConstants.veloright_rot[1]+theRotations[1])    # Y rotation
        rot.push_back(myConstants.veloright_rot[2]+theRotations[2])    # Z rotation

    # 3. Module misalignments
    
    elif (part_name[1:4] == 'Mod') :
        trans.push_back(myConstants.velomod_trans[0]+thePositions[0])  # X translation
        trans.push_back(myConstants.velomod_trans[1]+thePositions[1])  # Y translation
        trans.push_back(myConstants.velomod_trans[2]+(Z-origin)*scaleFactor+thePositions[2])  # Z translation
        rot.push_back(myConstants.velomod_rot[0]+theRotations[0])    # X rotation
        rot.push_back(myConstants.velomod_rot[1]+theRotations[1])    # Y rotation
        rot.push_back(myConstants.velomod_rot[2]+theRotations[2])    # Z rotation


    # 4. Detector misalignments
    
    elif (part_name[1:4] == 'Det') :
        trans.push_back(myConstants.velodet_trans[0]+thePositions[0])  # X translation
        trans.push_back(myConstants.velodet_trans[1]+thePositions[1])  # Y translation
        trans.push_back(myConstants.velodet_trans[2]+thePositions[2])  # Z translation
        rot.push_back(myConstants.velodet_rot[0]+theRotations[0])    # X rotation
        rot.push_back(myConstants.velodet_rot[1]+theRotations[1])    # Y rotation
        rot.push_back(myConstants.velodet_rot[2]+theRotations[2])    # Z rotation


    # 1. Global IT misalignments
    
    elif ( (part_name == '/ITSystem') or (part_name == '/OTSystem') ):
        trans.push_back(myConstants.tglo_trans[0]+thePositions[0])  # X translation
        trans.push_back(myConstants.tglo_trans[1]+thePositions[1])  # Y translation
        trans.push_back(myConstants.tglo_trans[2]+(Z-origin)*scaleFactor+thePositions[2])  # Z translation
        rot.push_back(myConstants.tglo_rot[0]+theRotations[0])  # X rotation
        rot.push_back(myConstants.tglo_rot[1]+theRotations[1])  # Y rotation
        rot.push_back(myConstants.tglo_rot[2]+theRotations[2])  # Z rotation

#        print myConstants.tglo_trans[0]
    # 3. Station misalignments
    
    elif ((part_name == '/ITT1') or (part_name == '/ITT2') or (part_name == '/ITT3')):
        trans.push_back(myConstants.tstat_trans[0]+thePositions[0])  # X translation
        trans.push_back(myConstants.tstat_trans[1]+thePositions[1])  # Y translation
        trans.push_back(myConstants.tstat_trans[2]+(Z-origin)*scaleFactor+thePositions[2])  # Z translation
        rot.push_back(myConstants.tstat_rot[0]+theRotations[0])  # X rotation
        rot.push_back(myConstants.tstat_rot[1]+theRotations[1])  # Y rotation
        rot.push_back(myConstants.tstat_rot[2]+theRotations[2])  # Z rotation

    elif ((part_name == '/T1Station') or (part_name == '/T2Station') or (part_name == '/T3Station')):
        trans.push_back(myConstants.tstat_trans[0]+thePositions[0])  # X translation
        trans.push_back(myConstants.tstat_trans[1]+thePositions[1])  # Y translation
        trans.push_back(myConstants.tstat_trans[2]+(Z-origin)*scaleFactor+thePositions[2])  # Z translation
        rot.push_back(myConstants.tstat_rot[0]+theRotations[0])  # X rotation
        rot.push_back(myConstants.tstat_rot[1]+theRotations[1])  # Y rotation
        rot.push_back(myConstants.tstat_rot[2]+theRotations[2])  # Z rotation

    else:
        trans.push_back(0.+thePositions[0])  # X translation
        trans.push_back(0.+thePositions[1])  # Y translation
        trans.push_back(0.+thePositions[2])  # Z translation
        rot.push_back(0.+theRotations[0])  # X rotation
        rot.push_back(0.+theRotations[1])  # Y rotation
        rot.push_back(0.+theRotations[2])  # Z rotation

    return transform

def isModule(name) :
    return name[1:4]=='Mod'
#------------------------------------------------------------------------------
# Write a cleaned-up version of the condition to the file conteining the XML
# catalogue of conditions.
#------------------------------------------------------------------------------
def writeCondition(alignCond) :

    return cleanString(alignCond.toXml())+'\n'

#------------------------------------------------------------------------------
# AlignmentCondition::toXml() returns a fully encapsulated, self consistent
# Xml string for the condition. We want to strip it of its encapsulation and
# put it inside an Xml catalogue.
#------------------------------------------------------------------------------
def cleanString(xmlString) :
    # cut some stuff out
    if (xmlString.find(xmlHeader()) != -1) :
        xmlString = xmlString.replace(xmlHeader(), '')

    if (xmlString.find(xmlTrailer()) != -1) :
        xmlString = xmlString.replace(xmlTrailer(), '')
    if (xmlString.find(condNameDef()) != -1) :
        xmlString = xmlString.replace(condNameDef(), condNameDef().replace('/',''))
    return xmlString

#------------------------------------------------------------------------------
# Remove header for single conditions
#------------------------------------------------------------------------------
def xmlHeader() :
    return '<?xml version="1.0" encoding="UTF-8"?><!DOCTYPE DDDB SYSTEM "structure.dtd"><DDDB>'
#------------------------------------------------------------------------------
# Remove trailing </DDDB> for single conditions
#------------------------------------------------------------------------------
def xmlTrailer() :
    return '</DDDB>'

#------------------------------------------------------------------------------
# Remove the leading '/' in condition name definition
#------------------------------------------------------------------------------
def condNameDef() :
    return 'name="/'

#------------------------------------------------------------------------------

def makeAlignments(topDetElem, myDet, condName, myDTD):
    
    print '************************************************************'
    print '    Modifying Alignment Constants in '+myDet+'              '
    print '************************************************************'


    
    global myXmlString
    myXmlString = '<?xml version="1.0" encoding="ISO-8859-1"?>\n<!DOCTYPE DDDB SYSTEM '+ myDTD + '>\n<DDDB>\n\n'
#    myXmlString = myXmlString + '<catalog name = "'+myDet+'">\n\n'
    DetElemDaughters(topDetElem, " |", condName)
#    myXmlString = myXmlString + '</catalog>\n' + '</DDDB>\n'
    myXmlString = myXmlString + '</DDDB>\n'

    return myXmlString

def writeAlignmentFile(file_name, myXmlString):

    if(myConstants.writeAlignFile == True):
        alignmentFile  = open(file_name,'w')
        alignmentFile.write(myXmlString)
        alignmentFile.close()


def makeMainCatalog(i):

    print '**********************************************'
    print '******** Generating MainCatalog.xml  *********'
    print '**********************************************'

    file_name = 'Conditions/MainCatalog_' + str(i) + '.xml'
    alignmentFile  = open(file_name,'w')
    alignmentFile.write('<?xml version="1.0" encoding="ISO-8859-1"?>\n<!DOCTYPE DDDB SYSTEM "../DTD/structure.dtd">\n<DDDB>\n')
    alignmentFile.write('\n')
    alignmentFile.write('  <catalog name ="Conditions">\n')
    alignmentFile.write('    <catalogref href ="#Alignment"/>\n')
    alignmentFile.write('    <catalogref href ="#Calibration"/>\n')
    alignmentFile.write('    <catalogref href ="#ChannelInfo"/>\n')
    alignmentFile.write('    <catalogref href ="#Environment"/>\n')
    alignmentFile.write('    <catalogref href ="#HardwareProperties"/>\n')
    alignmentFile.write('    <catalogref href ="#ReadoutConf"/>\n')
    alignmentFile.write('    <catalogref href ="Trigger/Catalog.xml#Trigger"/>\n')
    alignmentFile.write('  </catalog>\n')
    alignmentFile.write('\n')

    alignmentFile.write('  <catalog name="Alignment">\n')
    alignmentFile.write('    <catalogref href = "LHCb/AlignmentCatalog.xml#LHCb"/>\n')
#    alignmentFile.write('    <catalogref href = "Velo/AlignmentCatalog_' + str(i) + '.xml#Velo"/>\n')
    alignmentFile.write('    <catalogref href = "Velo/AlignmentCatalog.xml#Velo"/>\n')
    alignmentFile.write('    <catalogref href = "TT/AlignmentCatalog.xml#TT"/>\n')
    alignmentFile.write('    <catalogref href = "Rich1/AlignmentCatalog.xml#Rich1"/>\n')
    alignmentFile.write('    <catalogref href = "IT/AlignmentCatalog_' + str(i) + '.xml#IT"/>\n')
    alignmentFile.write('    <catalogref href = "OT/AlignmentCatalog_' + str(i) + '.xml#OT"/>\n')
    alignmentFile.write('    <catalogref href = "Rich2/AlignmentCatalog.xml#Rich2"/>\n')
    alignmentFile.write('    <catalogref href = "Prs/AlignmentCatalog.xml#Prs"/>\n')
    alignmentFile.write('    <catalogref href = "Ecal/AlignmentCatalog.xml#Ecal"/>\n')
    alignmentFile.write('    <catalogref href = "Hcal/AlignmentCatalog.xml#Hcal"/>\n')
    alignmentFile.write('    <catalogref href = "Muon/AlignmentCatalog.xml#Muon"/>\n')
    alignmentFile.write('  </catalog>\n')
    alignmentFile.write('\n')
    
    alignmentFile.write('  <catalog name="Calibration">\n')
    alignmentFile.write('    <catalogref href = "Spd/CalibrationCatalog.xml#Spd"/>\n')
    alignmentFile.write('    <catalogref href = "Prs/CalibrationCatalog.xml#Prs"/>\n')
    alignmentFile.write('    <catalogref href = "Ecal/CalibrationCatalog.xml#Ecal"/>\n')
    alignmentFile.write('    <catalogref href = "Hcal/CalibrationCatalog.xml#Hcal"/>\n')
    alignmentFile.write('    <catalogref href = "Muon/CalibrationCatalog.xml#Muon"/>\n')
    alignmentFile.write('  </catalog>\n')
    alignmentFile.write('\n')

    alignmentFile.write('  <catalog name="ChannelInfo">\n')
    alignmentFile.write('    <catalogref href="Velo/ChannelInfoCatalog.xml#Velo"/>\n')
    alignmentFile.write('  </catalog>\n')
    alignmentFile.write('\n')
    
    alignmentFile.write('  <catalog name="Environment">\n')
    alignmentFile.write('    <catalogref href = "LHCb/EnvironmentCatalog.xml#LHCb"/>\n')
    alignmentFile.write('    <catalogref href = "Rich1/EnvironmentCatalog.xml#Rich1"/>\n')
    alignmentFile.write('    <catalogref href = "Rich2/EnvironmentCatalog.xml#Rich2"/>\n')
    alignmentFile.write('  </catalog>\n')
    alignmentFile.write('\n')
    
    alignmentFile.write('  <catalog name="HardwareProperties">\n')
    alignmentFile.write('    <catalogref href="Velo/HardwarePropertiesCatalog.xml#Velo"/>\n')
    alignmentFile.write('  </catalog>\n')
    alignmentFile.write('\n')
    
    alignmentFile.write('  <catalog name="ReadoutConf">\n')
    alignmentFile.write('    <catalogref href = "Velo/ReadoutConfCatalog.xml#Velo"/>\n')
    alignmentFile.write('    <catalogref href = "TT/ReadoutConfCatalog.xml#TT"/>\n')
    alignmentFile.write('    <catalogref href = "IT/ReadoutConfCatalog.xml#IT"/>\n')
    alignmentFile.write('    <catalogref href = "Spd/ReadoutConfCatalog.xml#Spd"/>\n')
    alignmentFile.write('    <catalogref href = "Prs/ReadoutConfCatalog.xml#Prs"/>\n')
    alignmentFile.write('    <catalogref href = "Ecal/ReadoutConfCatalog.xml#Ecal"/>\n')
    alignmentFile.write('    <catalogref href = "Hcal/ReadoutConfCatalog.xml#Hcal"/>\n')
    alignmentFile.write('    <catalogref href = "Muon/ReadoutConfCatalog.xml#Muon"/>\n')
    alignmentFile.write('    <catalogref href = "Rich1/ReadoutConfCatalog.xml#Rich1"/>\n')
    alignmentFile.write('    <catalogref href = "Rich2/ReadoutConfCatalog.xml#Rich2"/>\n')
    alignmentFile.write('  </catalog>\n')
    alignmentFile.write('\n')

    alignmentFile.write('  <catalog name="Sim">\n')
    alignmentFile.write('  </catalog>\n')
    alignmentFile.write('\n')

    alignmentFile.write('</DDDB>\n')
    alignmentFile.close()

def makeVeloAlignmentCatalog(i):

    print '****************************************************'
    print '****** Generating VELO AlignmentCatalog.xml  *******'
    print '****************************************************'

    file_name = 'Conditions/Velo/AlignmentCatalog_' + str(i) + '.xml'
    veloAlignFile  = open(file_name,'w')
    veloAlignFile.write('<?xml version="1.0" encoding="ISO-8859-1"?>\n<!DOCTYPE DDDB SYSTEM "../../DTD/structure.dtd">\n<DDDB>\n')
    veloAlignFile.write('\n')
    veloAlignFile.write('  <catalog name ="Velo">\n')
    veloAlignFile.write('    <conditionref href = "Alignment/Global_' + str(i) + '.xml#VeloSystem"/>\n')
    veloAlignFile.write('    <conditionref href = "Alignment/Global_' + str(i) + '.xml#VeloRight"/>\n')
    veloAlignFile.write('    <conditionref href = "Alignment/Global_' + str(i) + '.xml#VeloLeft"/>\n')
    veloAlignFile.write('\n')
    
    veloAlignFile.write('    <conditionref href ="Alignment/Modules_' + str(i) + '.xml#ModulePU00"/>\n')
    veloAlignFile.write('    <conditionref href ="Alignment/Modules_' + str(i) + '.xml#ModulePU01"/>\n')
    veloAlignFile.write('    <conditionref href ="Alignment/Modules_' + str(i) + '.xml#ModulePU02"/>\n')
    veloAlignFile.write('    <conditionref href ="Alignment/Modules_' + str(i) + '.xml#ModulePU03"/>\n')
    for j in range(42):
        if (j < 10):
            veloAlignFile.write('    <conditionref href ="Alignment/Modules_' + str(i) + '.xml#Module0'+str(j)+'"/>\n')
        else:
            veloAlignFile.write('    <conditionref href ="Alignment/Modules_' + str(i) + '.xml#Module'+str(j)+'"/>\n')            

    veloAlignFile.write('\n')

    veloAlignFile.write('    <conditionref href ="Alignment/Detectors_' + str(i) + '.xml#DetectorPU00-00"/>\n')
    veloAlignFile.write('    <conditionref href ="Alignment/Detectors_' + str(i) + '.xml#DetectorPU01-00"/>\n')
    veloAlignFile.write('    <conditionref href ="Alignment/Detectors_' + str(i) + '.xml#DetectorPU02-00"/>\n')
    veloAlignFile.write('    <conditionref href ="Alignment/Detectors_' + str(i) + '.xml#DetectorPU03-00"/>\n')
    for j in range(42):
        if (j < 10):
            veloAlignFile.write('    <conditionref href ="Alignment/Detectors_' + str(i) + '.xml#Detector0'+str(j)+'-00"/>\n')
            veloAlignFile.write('    <conditionref href ="Alignment/Detectors_' + str(i) + '.xml#Detector0'+str(j)+'-01"/>\n')
        else:
            veloAlignFile.write('    <conditionref href ="Alignment/Detectors_' + str(i) + '.xml#Detector'+str(j)+'-00"/>\n')
            veloAlignFile.write('    <conditionref href ="Alignment/Detectors_' + str(i) + '.xml#Detector'+str(j)+'-01"/>\n')

    veloAlignFile.write('  </catalog>\n')
    veloAlignFile.write('</DDDB>\n')
    veloAlignFile.close()


def makeITAlignmentCatalog(i):

    print '****************************************************'
    print '****** Generating IT AlignmentCatalog.xml  *******'
    print '****************************************************'

    file_name = 'Conditions/IT/AlignmentCatalog_' + str(i) + '.xml'
    itAlignFile  = open(file_name,'w')
    itAlignFile.write('<?xml version="1.0" encoding="ISO-8859-1"?>\n<!DOCTYPE DDDB SYSTEM "../../DTD/structure.dtd">\n<DDDB>\n')
    itAlignFile.write('\n')
    itAlignFile.write('  <catalog name ="IT">\n')
    itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITSystem"/>\n')
    for j in range(1,4):
        itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'"/>\n')
        itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'TopBox"/>\n')
        itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'BottomBox"/>\n')
        itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'LeftBox"/>\n')
        itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'RightBox"/>\n')
        itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'TopLayerX1"/>\n')
        itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'TopLayerU"/>\n')
        itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'TopLayerV"/>\n')
        itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'TopLayerX2"/>\n') 
        itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'BottomLayerX1"/>\n')
        itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'BottomLayerU"/>\n')
        itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'BottomLayerV"/>\n')
        itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'BottomLayerX2"/>\n') 
        itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'LeftLayerX1"/>\n')
        itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'LeftLayerU"/>\n')
        itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'LeftLayerV"/>\n')
        itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'LeftLayerX2"/>\n') 
        itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'RightLayerX1"/>\n')
        itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'RightLayerU"/>\n')
        itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'RightLayerV"/>\n')
        itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'RightLayerX2"/>\n') 
        for k in range(1,8):
            itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'TopLayerX1Ladder'+str(k)+'"/>\n')
            itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'TopLayerULadder'+str(k)+'"/>\n')
            itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'TopLayerVLadder'+str(k)+'"/>\n')
            itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'TopLayerX2Ladder'+str(k)+'"/>\n') 
            itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'BottomLayerX1Ladder'+str(k)+'"/>\n')
            itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'BottomLayerULadder'+str(k)+'"/>\n')
            itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'BottomLayerVLadder'+str(k)+'"/>\n')
            itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'BottomLayerX2Ladder'+str(k)+'"/>\n') 
            itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'LeftLayerX1Ladder'+str(k)+'"/>\n')
            itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'LeftLayerULadder'+str(k)+'"/>\n')
            itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'LeftLayerVLadder'+str(k)+'"/>\n')
            itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'LeftLayerX2Ladder'+str(k)+'"/>\n') 
            itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'RightLayerX1Ladder'+str(k)+'"/>\n')
            itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'RightLayerULadder'+str(k)+'"/>\n')
            itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'RightLayerVLadder'+str(k)+'"/>\n')
            itAlignFile.write('    <conditionref href = "Alignment/Detectors_' + str(i) + '.xml#ITT'+str(j)+'RightLayerX2Ladder'+str(k)+'"/>\n') 

    itAlignFile.write('  </catalog>\n')
    itAlignFile.write('</DDDB>\n')

    itAlignFile.close()



def makeOTAlignmentCatalog(i):

    print '****************************************************'
    print '****** Generating OT AlignmentCatalog.xml  *******'
    print '****************************************************'

    file_name = 'Conditions/OT/AlignmentCatalog_' + str(i) + '.xml'
    otAlignFile  = open(file_name,'w')
    otAlignFile.write('<?xml version="1.0" encoding="ISO-8859-1"?>\n<!DOCTYPE DDDB SYSTEM "../../DTD/structure.dtd">\n<DDDB>\n')
    otAlignFile.write('\n')
    otAlignFile.write('  <catalog name ="OT">\n')
    otAlignFile.write('    <conditionref href = "Alignment/Elements_' + str(i) + '.xml#OTSystem"/>\n')
    for j in range(1,4):
        otAlignFile.write('    <conditionref href = "Alignment/Elements_' + str(i) + '.xml#T'+str(j)+'Station"/>\n')
        otAlignFile.write('    <conditionref href = "Alignment/Elements_' + str(i) + '.xml#T'+str(j)+'StationX1layer"/>\n')
        otAlignFile.write('    <conditionref href = "Alignment/Elements_' + str(i) + '.xml#T'+str(j)+'StationUlayer"/>\n')
        otAlignFile.write('    <conditionref href = "Alignment/Elements_' + str(i) + '.xml#T'+str(j)+'StationVlayer"/>\n')
        otAlignFile.write('    <conditionref href = "Alignment/Elements_' + str(i) + '.xml#T'+str(j)+'StationX2layer"/>\n')
        for k in range(0,4):
            for m in range(1,10):
                otAlignFile.write('    <conditionref href = "Alignment/Elements_' + str(i) + '.xml#T'+str(j)+'StationX1layerQuarter'+str(k)+'Module'+str(m)+'"/>\n')
        for k in range(0,4):
            for m in range(1,10):
                otAlignFile.write('    <conditionref href = "Alignment/Elements_' + str(i) + '.xml#T'+str(j)+'StationUlayerQuarter'+str(k)+'Module'+str(m)+'"/>\n')
        for k in range(0,4):
            for m in range(1,10):
                otAlignFile.write('    <conditionref href = "Alignment/Elements_' + str(i) + '.xml#T'+str(j)+'StationVlayerQuarter'+str(k)+'Module'+str(m)+'"/>\n')
        for k in range(0,4):
            for m in range(1,10):
                otAlignFile.write('    <conditionref href = "Alignment/Elements_' + str(i) + '.xml#T'+str(j)+'StationX2layerQuarter'+str(k)+'Module'+str(m)+'"/>\n')

    otAlignFile.write('  </catalog>\n')
    otAlignFile.write('</DDDB>\n')

    otAlignFile.close()


import gaudimodule
from ROOT import *
import myConstants
#-------------- Some convenient definitions -----------------
ROOT = gaudimodule.gbl.ROOT
std = gaudimodule.gbl.std
XYZPoint = ROOT.Math.XYZPoint
XYZVector = ROOT.Math.XYZVector

